@@ -1,4 +1,4 @@
-/*	$OpenBSD: main.c,v 1.298 2025/09/09 08:23:24 job Exp $ */
+/*	$OpenBSD: main.c,v 1.303 2025/12/31 12:51:52 tb Exp $ */
 /*
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -421,11 +421,21 @@ queue_add_from_mft(const struct mft *mft)
 
 	rp = repo_byid(mft->repoid);
 	for (i = 0; i < mft->filesz; i++) {
+		unsigned char *data = NULL;
+		size_t datasz = 0;
+
 		f = &mft->files[i];
 
 		if (f->type == RTYPE_INVALID || f->type == RTYPE_CRL)
 			continue;
 
+		if (f->type == RTYPE_CER) {
+			if ((data = calloc(1, sizeof(mft->mfthash))) == NULL)
+				err(1, NULL);
+			memcpy(data, mft->mfthash, sizeof(mft->mfthash));
+			datasz = sizeof(mft->mfthash);
+		}
+
 		if (mft->path != NULL)
 			if ((npath = strdup(mft->path)) == NULL)
 				err(1, NULL);
@@ -433,8 +443,9 @@ queue_add_from_mft(const struct mft *mft)
 			err(1, NULL);
 		if ((mftaki = strdup(mft->aki)) == NULL)
 			err(1, NULL);
-		entityq_add(npath, nfile, f->type, f->location, rp, NULL, 0,
-		    mft->talid, mft->certid, mftaki);
+
+		entityq_add(npath, nfile, f->type, f->location, rp,
+		    data, datasz, mft->talid, mft->certid, mftaki);
 	}
 }
 
@@ -634,10 +645,12 @@ entity_process(struct ibuf *b, struct validation_data *vd, struct stats *st)
 		cert = cert_read(b);
 		switch (cert->purpose) {
 		case CERT_PURPOSE_TA:
+			queue_add_from_cert(cert, &vd->ncas);
 			ccr_insert_tas(&vd->ccr.tas, cert);
-			/* FALLTHROUGH */
+			break;
 		case CERT_PURPOSE_CA:
 			queue_add_from_cert(cert, &vd->ncas);
+			ccr_insert_mft_sub(&vd->ccr.mfts, cert);
 			break;
 		case CERT_PURPOSE_BGPSEC_ROUTER:
 			cert_insert_brks(&vd->brks, cert);
@@ -681,8 +694,6 @@ entity_process(struct ibuf *b, struct validation_data *vd, struct stats *st)
 			repo_stat_inc(rp, talid, type, STYPE_INVALID);
 		roa_free(roa);
 		break;
-	case RTYPE_GBR:
-		break;
 	case RTYPE_ASPA:
 		io_read_buf(b, &ok, sizeof(ok));
 		if (ok == 0) {
@@ -803,7 +814,6 @@ sum_stats(const struct repo *rp, const struct repotalstats *in, void *arg)
 	out->aspas_invalid += in->aspas_invalid;
 	out->brks += in->brks;
 	out->crls += in->crls;
-	out->gbrs += in->gbrs;
 	out->taks += in->taks;
 	out->vrps += in->vrps;
 	out->vrps_uniqs += in->vrps_uniqs;
@@ -1178,6 +1188,8 @@ main(int argc, char *argv[])
 
 	check_fs_size(cachefd, cachedir);
 
+	x509_init_oid();
+
 	if (talsz == 0)
 		talsz = tal_load_default();
 	if (talsz == 0)
@@ -1475,9 +1487,15 @@ main(int argc, char *argv[])
 	 */
 
 	close(procfd);
+	msgbuf_free(procq);
 	close(rsyncfd);
+	msgbuf_free(rsyncq);
 	close(httpfd);
+	msgbuf_free(httpq);
 	close(rrdpfd);
+	msgbuf_free(rrdpq);
+	procq = rsyncq = httpq = rrdpq = NULL;
+	memset(queues, 0, sizeof(queues));
 
 	rc = 0;
 	for (;;) {
@@ -1540,6 +1558,8 @@ main(int argc, char *argv[])
 		timespecadd(&stats.system_time, &ts, &stats.system_time);
 	}
 
+	vd.buildtime = get_current_time();
+
 	/* change working directory to the output directory */
 	if (fchdir(outdirfd) == -1)
 		err(1, "fchdir output dir");
@@ -1588,7 +1608,6 @@ main(int argc, char *argv[])
 	    stats.repo_tal_stats.mfts, stats.repo_tal_stats.mfts_fail,
 	    stats.repo_tal_stats.mfts_gap);
 	printf("Certificate revocation lists: %u\n", stats.repo_tal_stats.crls);
-	printf("Ghostbuster records: %u\n", stats.repo_tal_stats.gbrs);
 	printf("Trust Anchor Keys: %u\n", stats.repo_tal_stats.taks);
 	printf("Repositories: %u\n", stats.repos);
 	printf("New files moved into validated cache: %u\n",
